﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using HostAnyControl.Annotations;
using Infragistics.Windows;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;

namespace HostAnyControl
{
    /// <summary>
    /// Interaction logic for MyWindow.xaml
    /// </summary>
    public partial class MyWindow : Window
    {
        public MyWindow()
        {
            InitializeComponent();

            var viewModelList = new ViewModelList();
            viewModelList.Add(new RowViewModel { Point1 = new Point(10, 20), Location1 = 12, Location2 = 12+2 });
            viewModelList.Add(new RowViewModel { Point1 = new Point(34, 54), Location1 = 25, Location2 = 25+2 });
            
            DataContext = viewModelList;
        }

        private void xamDG_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            this.ProcessCellValueModification();
        }

        void xamDG_EditModeEnded(object sender, EditModeEndedEventArgs e)
        {
            this.HideAdornerIfUnfocused();
        }

        void HideAdornerIfUnfocused()
        {
            if (this.AdornerIsVisible && !_adorner.IsKeyboardFocusWithin)
            {
                _adorner.Visibility = Visibility.Collapsed;
            }
        }

        bool AdornerIsVisible
        {
            get { return _adorner != null && _adorner.Visibility == Visibility.Visible; }
        }

        #region Data

        UIElementAdorner<PointEditControl> _adorner;


        void CreateProxyForAdorner()
        {
            if (_adorner == null || this.xamDG.ActiveCell == null)
                return;

            var numbersViewModel = (RowViewModel)this.xamDG.ActiveCell.Record.DataItem;
            //_adorner.DataContext = numbersViewModel.Point1;
            _adorner.Child.ViewModel = numbersViewModel;
        }
        /// <summary>
        /// Method to show the Adorner
        /// </summary>
        void ProcessCellValueModification()
        {
            if (_adorner == null && !this.BuildAdorner())
                return;

            Debug.Assert(_adorner != null, "_adorner should exist now.");

            this.ActivateAdorner();
        }

        void ActivateAdorner()
        {
            this.CreateProxyForAdorner();
            this.MoveAdornerToActiveCell();
        }



        bool BuildAdorner()
        {
            if (_adorner != null)
                return false;

            AdornerLayer layer = AdornerLayer.GetAdornerLayer(this.xamDG);
            if (layer == null)
                return false;

            _adorner = new UIElementAdorner<PointEditControl>(this.xamDG, new PointEditControl(this.xamDG));
            layer.Add(_adorner);
            return true;
        }

        void MoveAdornerToActiveCell()
        {
            if (_adorner == null || this.xamDG.ActiveCell == null)
                return;

            CellValuePresenter cvp = this.FindCellValuePresenterForActiveCell();
            if (cvp != null)
            {
                Point location = this.CalculateAdornerLocation(cvp);
                _adorner.SetOffsets(location.X, location.Y);
            }
        }

        CellValuePresenter FindCellValuePresenterForActiveCell()
        {
            if (this.xamDG.ActiveCell == null)
                return null;

            FrameworkElement fe = Keyboard.FocusedElement as FrameworkElement;
            if (fe == null)
                return null;

            CellValuePresenter cvp = fe as CellValuePresenter;

            if (cvp == null)
                cvp = Utilities.GetAncestorFromType(fe, typeof(CellValuePresenter), true) as CellValuePresenter;

            if (cvp == null || !cvp.IsDescendantOf(this.xamDG))
                return null;

            return cvp;
        }

        Point CalculateAdornerLocation(CellValuePresenter cvp)
        {
            GeneralTransform trans = cvp.TransformToAncestor(this.xamDG);
            Point location = trans.Transform(new Point(0, 0));

            this.VerifyAdornerMetrics();

            // Make sure the adorner does not extend past the grid's bottom edge.
            double bottomOfAdorner = location.Y + cvp.ActualHeight + _adorner.ActualHeight;
            if (this.xamDG.ActualHeight <= bottomOfAdorner)
                location.Offset(0, -_adorner.ActualHeight);
            else
                location.Offset(0, cvp.ActualHeight);

            return location;
        }

        void VerifyAdornerMetrics()
        {
            if (_adorner == null)
                return;

            if (_adorner.Visibility != Visibility.Visible)
                _adorner.Visibility = Visibility.Visible;

            if (_adorner.ActualHeight < 1)
            {
                _adorner.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));
                _adorner.Arrange(new Rect(new Point(0, 0), _adorner.DesiredSize));
            }
        }
        #endregion // Data


    }

    public class ViewModelList : ObservableCollection<RowViewModel>
    {
    }

    public class RowViewModel : INotifyPropertyChanged
    {
        public double Location1
        {
            get { return location1; }
            set
            {
                if (value.Equals(location1)) return;
                location1 = value;
                OnPropertyChanged("Location1");
            }
        }

        public double Location2
        {
            get { return location2; }
            set
            {
                if (value.Equals(location2)) return;
                location2 = value;
                OnPropertyChanged("Location2");
            }
        }

        private Point point1;
        private double location1;
        private double location2;

        public Point Point1
        {
            get { return point1; }
            set
            {
                if (value.Equals(point1)) return;
                point1 = value;
                OnPropertyChanged("Point1");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
